#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include <cstdlib>
#include <cassert>

using namespace std;

class Weapon;

typedef enum {SWORD, AXE} WeaponType;
typedef enum {SPIDER, TROLL} MonsterType;


class Monster {
protected:
    Monster():maxHealth(100+rand()%156){ health=maxHealth; }
    Monster(const string & name_, int health_, int maxHealth_):health(health_),maxHealth(maxHealth_),name(name_){ ; }

    int health;
    int maxHealth;
    string name;
public:
    string getName() const { return name; }
    void setName(const string & name_){ name = name_; }
    int getHealth() const { return health; }
    void setHealth(int h){ health = h; }
    int getMaxHealth() const { return maxHealth; }
    void setMaxHealth(int h){ maxHealth = h; }

    virtual void receiveAttack(const Weapon * w, WeaponType t) = 0;

    virtual std::ostream & print(std::ostream & out) const = 0;
};

class Weapon {
protected:
    int damage;

    Weapon():damage(rand()%20 + 1){ ; }
    Weapon(int d):damage(d){ ; }
public:
    virtual int getDamage(MonsterType mt) const = 0;
};

struct Troll: public Monster {
    Troll(){ setName("Brick"); }
    Troll(const string & name_, int health_, int maxHealth_):Monster(name_, health_, maxHealth_) { ; }

    virtual void receiveAttack(const Weapon * w, WeaponType t) {
		if (t == AXE) {
			std::stringstream s;
			s << health << " / " << maxHealth;
			setName(s.str());
		}
		health -= w->getDamage(TROLL);
	}

    virtual std::ostream & print(std::ostream & out) const {
        return out << "troll" << " \"" << getName() << "\" " << getHealth() << " / " << getMaxHealth();
    }
};

struct Spider: public Monster {
    Spider(){ setName("Odula"); }
    Spider(const string & name_, int health_, int maxHealth_):Monster(name_, health_, maxHealth_) { ; }

    virtual void receiveAttack(const Weapon * w, WeaponType t) {
		health -= w->getDamage(SPIDER);
	}

    virtual std::ostream & print(std::ostream & out) const {
        return out << "spider" << " \"" << getName() << "\" " << getHealth() << " / " << getMaxHealth();
    }
};

std::ostream & operator<<(std::ostream & out, const Monster & s){
    return s.print(out);
}

std::istream & operator>>(std::istream & in, Monster & m){
    std::string tmpLine;
    std::getline(in, tmpLine);

    size_t qtStart = tmpLine.find_first_of("\"") + 1; // swallow start "
    size_t qtEnd = tmpLine.find_last_of("\"");

    string name = tmpLine.substr(qtStart, qtEnd - qtStart);
    m.setName(name);

    std::stringstream tmpIn;
    tmpIn.str(tmpLine.substr(qtEnd + 1)); // swallow end "

    int health;
    tmpIn >> health;

    char slash;
    tmpIn >> slash;

    int maxHealth;
    tmpIn >> maxHealth;

    m.setMaxHealth(maxHealth);
    m.setHealth(health);

    return in;
}

struct Axe: public Weapon {
    Axe(int d):Weapon(d){;}

    virtual int getDamage(MonsterType mt) const {
        switch (mt) {
        case SPIDER: return 0.2*damage;
        case TROLL: return 0.6*damage;
        }
        return 0;
    }
};

struct Sword: public Weapon {
    Sword(int d):Weapon(d){;}

    virtual int getDamage(MonsterType mt) const {
        switch (mt) {
        case SPIDER: return 0.4*damage;
        case TROLL: return 0.7*damage;
        }
        return 0;
    }
};



int main()
{
	fstream zdroj("../05_in_monsters.txt", std::ios_base::in);
	fstream slabi("../slabi.txt", std::ios_base::out);

	while (zdroj.good()) {
		std::string line;
		std::getline(zdroj, line);
		if (!zdroj.good()) continue;

		std::stringstream lineStream;
		lineStream.str(line);

		std::string race;
		lineStream >> race;

		Monster * m = NULL;

		if (race == "troll") {
			m = new Troll;
		}
		else if (race == "spider") {
			m = new Spider;
		}

		assert(m != NULL);

		lineStream >> *m;

		if (m->getHealth() < 60) {
			slabi << *m << std::endl;
		}
	}

    return 0;
}

